import inspect

from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.module_loading import import_module

from post_office import models

from plugs_core import utils
from plugs_mail.mail import PlugsMail


class Command(BaseCommand):

    def handle(self, *args, **options):
        templates = self.get_apps()
        self.create_templates(templates)

    def get_apps(self):
        """
        Get the list of installed apps
        and return the apps that have
        an emails module
        """
        templates = []
        for app in settings.INSTALLED_APPS:
            try:
                app = import_module(app + '.emails')
                templates += self.get_plugs_mail_subs(app)
            except ImportError:
                pass
        return templates

    def get_plugs_mail_subs(self, app):
        """
        Returns a list of tuples, but it should
        return a list of dicts
        """
        classes = []
        members = inspect.getmembers(app)
        for member in members:
            name, cls = member
            if inspect.isclass(cls) and issubclass(cls, PlugsMail) and name != 'PlugsMail':
                try:
                    subject = cls.subject
                    description = cls.description
                    location = app.__file__
                    classes.append((name, location, subject, description))
                except AttributeError:
                    raise AttributeError('Email class must specify email subject and description.')
        return classes

    def create_templates(self, templates):
        """
        Gets a list of templates to insert into the database
        """
        for template in templates:
            name, location, subject, description = template
            if not self.template_exists_db(name):
                # create template if it does not exists
                # if does not exist, try to find a default template
                dir_ = location[:-9] + 'templates/emails/'
                file_ = dir_ + utils.camel_to_snake(name) + '.html'
                text = self.open_file(file_)
                data = {
                    'name': utils.camel_to_snake(name).upper(),
                    'html_content': text,
                    'content': self.text_version(text),
                    'subject': subject,
                    'description': description
                }
                models.EmailTemplate.objects.create(**data)

    def text_version(self, html):
        """
        Uses util to create a text email template
        from a html one
        """
        return utils.html_to_text(html)

    def open_file(self, file_):
        """
        Receives a file path has input and returns a
        string with the contents of the file
        """
        with open(file_, 'r', encoding='utf-8') as file:
            text = ''
            for line in file:
                text += line
        return text

    def template_exists_db(self, template):
        """
        Receives a template name and sees if it exists in the database
        """
        template = utils.camel_to_snake(template).upper()
        try:
            models.EmailTemplate.objects.get(name=template)
        except models.EmailTemplate.DoesNotExist:
            return False
        return True



"""
Plugs Mail Settings
"""

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


MANDATORY_SETTINGS = ['SEND_EMAILS']
PROJECT_SETTINGS = getattr(settings, 'PLUGS_MAIL', {})


for setting in MANDATORY_SETTINGS:
    try:
        PROJECT_SETTINGS[setting]
    except KeyError:
        raise ImproperlyConfigured('Missing setting: PLUGS_MAIL[\'{0}\']'.format(setting))


app_settings = PROJECT_SETTINGS



"""
Plugs Mail Utils
"""

def to_email(email_class, email, **data):
    """
    Send email to specified email address
    """
    email_class().send([email], **data)

def to_user(email_class, user, **data):
    """
    Email user
    """
    email_class().send([user.email], **data)

def to_staff(email_class, **data):
    """
    Email staff users
    """
    for user in get_user_model().objects.filter(is_staff=True):
        email_class().send([user.email], **data)

def to_superuser(email_class, **data):
    """
    Email superusers
    """
    for user in get_user_model().objects.filter(is_superuser=True):
        email_class().send([user.email], **data)



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='plugs-mail',
    package = 'plugs_mail',
    version='0.1.2',
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    description='Reusable Mail APP',
    long_description=README,
    url='https://github.com/yo-solo/plugs-mail',
    author='Ricardo Lobo',
    author_email='ricardolobo@soloweb.pt',
    install_requires = [
        'plugs-core>=0.1.0',
        'django-post-office>=2.0.7',
        'cssselect>=1.0.0'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
